#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Script for reading in the solvent XML, and to extract the concentration of
each molecule in a solvent mixture, and the number of SSIPs corresponding to
each molecule.

This allows the calculation of the fractional occupancy of the phase.

@author: mark
"""

import logging
from lxml import etree

logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.WARN)

PHASE_NAMESPACE_DICT = {"phase":"http://www-hunter.ch.cam.ac.uk/PhaseSchema",
                        "ssip":"http://www-hunter.ch.cam.ac.uk/SSIP",
                        'cml':'http://www.xml-cml.org/schema'}

IDEAL_MAX_SSIP_CONCENTRATION_MOLAR = 300.0

def get_fractional_occupancy_for_solvent_from_file(solvent_filename, solvent_id):
    """This reads inthe file and extracts the solvent information, and returns
    the fractional occupancy contribution from the solvent.
    """
    solvent_list_element = read_element_tree(solvent_filename)
    return get_fractional_occupancy_for_solvent(solvent_list_element, solvent_id)

def get_fractional_occupancy_for_solvent(solvent_list_element, solvent_id):
    """This extracts the solvent XML, and calculates the fractional occupancy
    contribution for the solvent.
    """
    solvent_element = get_solvent_element_by_id(solvent_list_element, solvent_id)
    return calculate_fractional_occupancy_for_solvent(solvent_element)

def calculate_fractional_occupancy_for_solvent(solvent_element):
    """This calculates the contribution to the fractional occupancy from the solvent.
    """
    xpath_expression = "phase:Molecule"
    molecule_element_list = solvent_element.xpath(xpath_expression, namespaces=PHASE_NAMESPACE_DICT)
    solvent_fractional_occupancy = 0.0
    for molecule_element in molecule_element_list:
        fractional_occupancy_contribution = calculate_fractional_occupancy_contribution(molecule_element)
        solvent_fractional_occupancy += fractional_occupancy_contribution
    return solvent_fractional_occupancy

def calculate_fractional_occupancy_contribution(molecule_element):
    """This calculates the contribution to the fractional occupancy for this species.
    """
    concentration_value = get_concentration_value(molecule_element)
    number_of_ssips = get_number_of_ssips_in_molecule(molecule_element)
    return (concentration_value * number_of_ssips)/IDEAL_MAX_SSIP_CONCENTRATION_MOLAR
    
def get_concentration_value(molecule_element):
    """This extracts the concentration.
    """
    xpath_expression = "phase:Concentration"
    concentration_element_list = molecule_element.xpath(xpath_expression, namespaces=PHASE_NAMESPACE_DICT)
    if len(concentration_element_list) == 1:
        return float(concentration_element_list[0].text)
    else:
        LOGGER.debug("len:")
        LOGGER.debug(len(concentration_element_list))
        LOGGER.debug(concentration_element_list)
        raise ValueError("Wrong number of Concentration Elements.")

def get_number_of_ssips_in_molecule(molecule_element):
    """This extracts the SSIP elements and return the number.
    """
    xpath_expression = "ssip:SSIPList/ssip:SSIP"
    ssip_element_list = molecule_element.xpath(xpath_expression, namespaces=PHASE_NAMESPACE_DICT)
    return len(ssip_element_list)

def get_solvent_element_by_id(solvent_list_element, solvent_id):
    """This extracts the solvent element from the information
    """
    xpath_expression = create_solvent_xpath_expression(solvent_id)
    solvent_element_list = solvent_list_element.xpath(xpath_expression, namespaces=PHASE_NAMESPACE_DICT)
    if len(solvent_element_list) == 1:
        return solvent_element_list[0]
    else:
        LOGGER.debug("len:")
        LOGGER.debug(len(solvent_element_list))
        LOGGER.debug(solvent_element_list)
        raise ValueError("Wrong number of Solvent Elements.")

def create_solvent_xpath_expression(solvent_id):
    """This creates the xpath expression to extract the solvent Element based
    on solventID.
    """
    return 'phase:Solvent[@phase:solventID="{:s}"]'.format(solvent_id)

def read_element_tree(filename):
    """This reads in an xml file to an element tree.
    """
    return etree.parse(filename)
